package org.diagbot.graphWeb.work;

import com.alibaba.fastjson.JSONObject;
import org.diagbot.common.javabean.MangementEvaluation;
import org.diagbot.common.work.FeatureRate;
import org.diagbot.common.javabean.Filnlly;
import org.diagbot.common.work.ResponseData;
import org.diagbot.common.work.SearchData;
import org.diagbot.graph.jdbc.DriverManager;
import org.diagbot.graph.jdbc.Neo4jAPI;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

import org.diagbot.common.javabean.MedicalIndication;
import org.diagbot.common.javabean.MedicalIndicationDetail;
import org.diagbot.graphWeb.util.MapValueComparator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class GraphCalculate {
    Logger logger = LoggerFactory.getLogger(GraphCalculate.class);
    /**
     * 返回诊断和治疗
     * @param request
     * @param searchData
     * @return
     * @throws Exception
     */
    public ResponseData calculate(HttpServletRequest request, SearchData searchData) throws Exception {
        ResponseData responseData = new ResponseData();
        ParamsDataProxy paramsDataProxy = new ParamsDataProxy();
        paramsDataProxy.createSearchData(request,searchData);
        List<String> inputList = new ArrayList<>();
        Map<String, Map<String, String>> inputs = searchData.getInputs();
        Set<String> ss = inputs.keySet();
        inputList.addAll(ss);
        Neo4jAPI neo4jAPI = new Neo4jAPI(DriverManager.newDrive());
        //计算诊断
        Map<String, String> condition = neo4jAPI.getCondition((String[]) inputList.toArray(new String[inputList.size()]));
        List<FeatureRate> featureRates = new ArrayList<>();
        for (Map.Entry<String,String> d:condition.entrySet()) {
            if(!"低血糖反应".equals(d.getKey()) && !"胃肠道不良反应".equals(d.getKey())
                    && !"肾功能不全".equals(d.getKey())){
                FeatureRate featureRate = new FeatureRate();
                featureRate.setFeatureName(d.getKey());
                featureRate.setExtraProperty("");
                featureRate.setDesc(d.getValue());
                featureRate.setRate("neo4j");
                featureRates.add(featureRate);
            }
        }
        String webDiag = searchData.getDiag();
        Set<String> diseaseSet = condition.keySet();
        String[] featureTypes = searchData.getFeatureTypes();
        List<String> featureTypeList = Arrays.asList(featureTypes);
        logger.info("featureTypeList : "+featureTypeList);
        Integer diseaseType = searchData.getDisType();
        //走治疗
        if(webDiag.trim()!=null && webDiag.trim()!="" && featureTypeList.contains("8")){
            //走平常诊断治疗
            Map<String, Filnlly> mulDiseaseTreat = neo4jAPI.getMulDiseaseTreat_2(webDiag,diseaseType,diseaseSet,String.join(",", inputList));
            responseData.setTreat(mulDiseaseTreat);
        }
        //管理评估(慢病才有)
        if(featureTypeList.contains("11") && diseaseType == 1){
            logger.info("featureTypeList 包含11,走管理评估!!!");
            MangementEvaluation mangementEvaluation = this.getMangementEvaluation();
            Map<String, JSONObject> mangementEvaluation1 = mangementEvaluation.getMangementEvaluation();
            responseData.setManagementEvaluation(mangementEvaluation1);
        }
        //指标推送
        if(featureTypeList.contains("22") && diseaseSet != null && diseaseSet.size()>0){
            logger.info("featureTypeList 包含22,走指标推送!!!,图谱推出的诊断为:"+diseaseSet);
            List<MedicalIndication> medicalIndicationList = new ArrayList<>();
            MedicalIndication medicalIndication = this.getMedicalIndication(diseaseSet);
            medicalIndicationList.add(medicalIndication);
            responseData.setMedicalIndications(medicalIndicationList);
        }
        //诊断推送
        responseData.setDis(featureRates);
        responseData.setInputs(searchData.getInputs());
        return responseData;
    }

    /**
     * 返回LIS,PACS
     * @param searchData
     */
    public ResponseData getLisPacs(HttpServletRequest request,SearchData searchData) throws Exception {
        ResponseData responseData = new ResponseData();
        Neo4jAPI neo4jAPI = new Neo4jAPI(DriverManager.newDrive());
        String webDiag = searchData.getDiag();
        List<String> webDiagList = Arrays.asList(webDiag.split(",|，|、"));
        List<FeatureRate> bigdataDiagFeature = searchData.getPushDiags();
        List<String> bigdataDiagList =new LinkedList<>();
        if(bigdataDiagFeature.size()>0){
            for (FeatureRate fe:bigdataDiagFeature) {
                if("neo4j".equals(fe.getRate())){
                    bigdataDiagList.add(fe.getFeatureName());
                    logger.info("图谱推出的诊断为: "+fe.getFeatureName());
                }else {
                    bigdataDiagList.add(fe.getFeatureName());
                    logger.info("大数据推出的诊断为: "+fe.getFeatureName());
                }
            }
        }
        for (String web: webDiagList ) {
                for (int i=0;i<bigdataDiagList.size();i++ ) {
                    if(bigdataDiagList.get(i).equals(web)){
                        bigdataDiagList.remove(bigdataDiagList.get(i));
                    }
                }
        }
        logger.info("界面诊断为: " + webDiagList);
        logger.info("推出的诊断合并为: " + bigdataDiagList);
        Map<String, Set<String>> weblisPacs1 =null;
        Map<String, Set<String>> biglisPacs1 =null;
        if(webDiagList !=null && webDiagList.size()>0){
            weblisPacs1=neo4jAPI.getLisPacs(webDiagList);//界面诊断推出的LIS,PACS
        }
        if(bigdataDiagList !=null && bigdataDiagList.size()>0){
            biglisPacs1=neo4jAPI.getLisPacs(bigdataDiagList);//大数据推得诊断
        }
        Set<String> lis =null;
        Set<String> pacs =null;
        //如果界面有诊断
        if(weblisPacs1 !=null && weblisPacs1.values().size()>0){
            lis = weblisPacs1.get("LIS");
            pacs = weblisPacs1.get("PACS");
            logger.info("界面有诊断的情况下,界面诊断推出的lis为: "+lis);
            logger.info("界面有诊断的情况下,界面诊断推出的pacs为: "+pacs);
            if(biglisPacs1 !=null && biglisPacs1.values().size()>0){
                Set<String> bl = biglisPacs1.get("LIS");
                Set<String>  bp= biglisPacs1.get("PACS");
                logger.info("界面有诊断的情况下,推出诊断的lis为: "+bl);
                logger.info("界面有诊断的情况下,推出诊断的pacs为: "+bp);
                lis.addAll(bl);
                pacs.addAll(bp);
            }
        }else {
            lis = biglisPacs1.get("LIS");
            pacs = biglisPacs1.get("PACS");
            logger.info("界面无诊断的情况下,推出诊断的lis为: "+lis);
            logger.info("界面无诊断的情况下,推出诊断的lis为: "+pacs);
        }
        logger.info("推出的合并lis为: " + lis);
        logger.info("推出的合并pacs为: " + pacs);
        ArrayList<FeatureRate> lisFeature = new ArrayList<>();
        ArrayList<FeatureRate> pacsFeature = new ArrayList<>();
        for (String l:lis) {
            FeatureRate featureRate = new FeatureRate();
            featureRate.setFeatureName(l);
            lisFeature.add(featureRate);
        }
        for (String p:pacs) {
            FeatureRate featureRate = new FeatureRate();
            featureRate.setFeatureName(p);
            pacsFeature.add(featureRate);
        }
        responseData.setLabs(lisFeature);
        responseData.setPacs(pacsFeature);
        return responseData;
    }


    /**
     * 对化验和检查的结果进行排序
     * @param set
     * @return List<FeatureRate>
     */
    public Set<String> processResult(Set<String> set) {
        Set<String> frlist = new LinkedHashSet<>();
        Map<String, String> sortval = new HashMap<>();
        Map<String, String> items = new HashMap<>();
        String name;
        try {
            if (set != null && set.size() > 0) {
                for (String item : set) {
                    if (sortval.get(item) == null) {
                        sortval.put(item, "1");
                    } else {
                        sortval.put(item, String.valueOf(Integer.parseInt(sortval.get(item)) + 1));
                    }
                }
                sortval = sortMapByValue(sortval);
                for (String key : sortval.keySet()) {
                    frlist.add(items.get(key));
                }
            }
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
        finally {
            return frlist;
        }
    }


    /**
     * 使用 Map按value进行排序
     * @param oriMap
     * @return
     */
    public static Map<String, String> sortMapByValue(Map<String, String> oriMap) {
        if (oriMap == null || oriMap.isEmpty()) {
            return null;
        }
        Map<String, String> sortedMap = new LinkedHashMap<String, String>();
        List<Map.Entry<String, String>> entryList = new ArrayList<Map.Entry<String, String>>(
                oriMap.entrySet());
        Collections.sort(entryList, new MapValueComparator());
        Iterator<Map.Entry<String, String>> iter = entryList.iterator();
        Map.Entry<String, String> tmpEntry = null;
        while (iter.hasNext()) {
            tmpEntry = iter.next();
            sortedMap.put(tmpEntry.getKey(), tmpEntry.getValue());
        }
        return sortedMap;
    }

    /**
     * 量表和指标
     * @param diseaseSet
     * @return
     */
    public MedicalIndication getMedicalIndication(Set diseaseSet){
        MedicalIndication medicalIndication = new MedicalIndication();
        if(diseaseSet.contains("肾功能不全")){
            logger.info("diseaseSet中包含肾功能不全");
            //推送指标
            medicalIndication.setName("肾功能不全");
            List<MedicalIndicationDetail> medicalIndicationDetailList = new ArrayList<>();
            //量表
            MedicalIndicationDetail medicalIndicationDetail1 = new MedicalIndicationDetail();
            medicalIndicationDetail1.setType(1);
            JSONObject liangJson = new JSONObject();
            liangJson.put("name","密西根糖尿病周围神经病评分（MDNS）");
            medicalIndicationDetail1.setContent(liangJson);
            //公式
            MedicalIndicationDetail medicalIndicationDetail2 = new MedicalIndicationDetail();
            medicalIndicationDetail2.setType(2);
            JSONObject gongshiJson = new JSONObject();
            gongshiJson.put("name","肾小球滤过率");
            List<JSONObject> detailList = new ArrayList<>();
            JSONObject ageJsonObject = new JSONObject();
            ageJsonObject.put("controlType",2);
            ageJsonObject.put("name","年龄");
            ageJsonObject.put("value","");
            ageJsonObject.put("uint","");
            ageJsonObject.put("details",new ArrayList<>());
            JSONObject indicationJsonObject = new JSONObject();
            indicationJsonObject.put("controlType",2);
            indicationJsonObject.put("name","血肌酐");
            indicationJsonObject.put("value","");
            indicationJsonObject.put("uint","mg/dL");
            indicationJsonObject.put("details",new ArrayList<>());
            JSONObject sexJson = new JSONObject();
            sexJson.put("controlType",3);
            sexJson.put("name","性别");
            sexJson.put("value","");
            sexJson.put("uint","");
            List<JSONObject> sexList = new ArrayList<>();
            JSONObject manJson = new JSONObject();
            manJson.put("detailName","男");
            manJson.put("value",1);
            manJson.put("state",0);
            JSONObject womanJson = new JSONObject();
            womanJson.put("detailName","女");
            womanJson.put("value",1.018);
            womanJson.put("state",1);
            sexList.add(manJson);
            sexList.add(womanJson);
            sexJson.put("details",sexList);
            detailList.add(ageJsonObject);
            detailList.add(indicationJsonObject);
            detailList.add(sexJson);
            gongshiJson.put("details",detailList);
            JSONObject resultJson = new JSONObject();
            resultJson.put("value","");
            resultJson.put("unit","");
            resultJson.put("text","");
            gongshiJson.put("result",resultJson);
            medicalIndicationDetail2.setContent(gongshiJson);
            //其他
            MedicalIndicationDetail medicalIndicationDetail3 = new MedicalIndicationDetail();
            medicalIndicationDetail3.setType(3);
            JSONObject shiFouJson = new JSONObject();
            shiFouJson.put("controlType",0);
            List<JSONObject> shiFouList = new ArrayList<>();
            JSONObject shiJsonObject = new JSONObject();
            shiJsonObject.put("detailName","是");
            shiJsonObject.put("state",0);
            JSONObject fouJsonObject = new JSONObject();
            fouJsonObject.put("detailName","否");
            fouJsonObject.put("state",1);
            shiFouList.add(shiJsonObject);
            shiFouList.add(fouJsonObject);
            shiFouJson.put("details",shiFouList);
            medicalIndicationDetail3.setContent(shiFouJson);
            medicalIndicationDetailList.add(medicalIndicationDetail1);
            medicalIndicationDetailList.add(medicalIndicationDetail2);
            medicalIndicationDetailList.add(medicalIndicationDetail3);
            medicalIndication.setDetails(medicalIndicationDetailList);
        }
        return medicalIndication;
    }

    /**
     * 管理评估
     * @return
     */
    public MangementEvaluation getMangementEvaluation(){
        MangementEvaluation mangementEvaluation = new MangementEvaluation();
        Map<String,JSONObject> mangementMap = new HashMap<>();
        JSONObject curativeJson = new JSONObject();
        List<JSONObject> itemJson = new ArrayList<>();
        JSONObject bloodPressure = new JSONObject();
        bloodPressure.put("controltype",0);
        bloodPressure.put("name","血压控制情况");
        List<JSONObject> bloodPressureLevelList = new ArrayList<>();
        String[] items= {"正常","1级","2级","3级"};
        for (String item:items) {
            JSONObject iJson = new JSONObject();
            iJson.put("detailName",item);
            iJson.put("state",0);
            bloodPressureLevelList.add(iJson);
        }
        bloodPressure.put("details",bloodPressureLevelList);
        itemJson.add(bloodPressure);

        JSONObject bmiControl = new JSONObject();
        bmiControl.put("controltype",0);
        bmiControl.put("name","BMI控制情况");
        List<JSONObject> bmiControlLevelList = new ArrayList<>();
        String[] bmiControlitems= {"体重过低","体重达标","肥胖前期","肥胖"};
        for (String item:bmiControlitems) {
            JSONObject iJson = new JSONObject();
            iJson.put("detailName",item);
            iJson.put("state",0);
            bmiControlLevelList.add(iJson);
        }
        bmiControl.put("details",bmiControlLevelList);
        itemJson.add(bmiControl);

        JSONObject bloodFatControl = new JSONObject();
        bloodFatControl.put("controltype",0);
        bloodFatControl.put("name","血脂控制情况");
        List<JSONObject> bloodFatControlLevelList = new ArrayList<>();
        String[] bloodFatControlitems= {"高血脂"};
        for (String item:bloodFatControlitems) {
            JSONObject iJson = new JSONObject();
            iJson.put("detailName",item);
            iJson.put("state",0);
            bloodFatControlLevelList.add(iJson);
        }
        bloodFatControl.put("details",bmiControlLevelList);
        itemJson.add(bloodFatControl);
        curativeJson.put("rows",itemJson);
        //分级管理
        JSONObject gradeJson = new JSONObject();
        List<JSONObject> gradItemJson = new ArrayList<>();
        JSONObject bigGrad = new JSONObject();
        bigGrad.put("name","血压控制情况");
        bigGrad.put("controltype",0);
        List<JSONObject> gradItemJ = new ArrayList<>();
        String[] gradItem ={"常规管理","强化管理"};
        for (String item:gradItem) {
            JSONObject j =new JSONObject();
            j.put("detailName",item);
            j.put("state",0);
            gradItemJ.add(j);
        }
        bigGrad.put("details",gradItemJ);
        gradItemJson.add(bigGrad);
        gradeJson.put("rows",gradItemJson);
        mangementMap.put("疗效评估",curativeJson);
        mangementMap.put("糖尿病分级管理",gradeJson);
        mangementEvaluation.setMangementEvaluation(mangementMap);
        return mangementEvaluation;
    }
}
