package org.diagbot.service.impl;

import org.diagbot.entity.node.LIS;
import org.diagbot.entity.node.Symptom;
import org.diagbot.mapper.KnowledgeMapper;
import org.diagbot.service.KnowledgeService;
import org.diagbot.vo.*;
import org.diagbot.entity.node.Disease;
import org.diagbot.entity.node.base.BaseNode;
import org.apache.commons.collections4.IteratorUtils;
import org.diagbot.repository.*;
import org.diagbot.vo.domain.*;
import org.neo4j.driver.v1.types.Node;
import org.neo4j.driver.v1.types.Path;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;

import java.math.RoundingMode;
import java.text.NumberFormat;
import java.util.*;


@Service
public class KnowledgeServiceImpl implements KnowledgeService {
    Logger logger = LoggerFactory.getLogger(KnowledgeServiceImpl.class);
    @Autowired
    private SymptomRepository symptomRepository;
    @Autowired
    private DiseaseRepository diseaseRepository;
    @Autowired
    private LISRepository lisRepository;
    @Autowired
    private BaseNodeRepository baseNodeRepository;
    @Autowired
    private RelationRepository relationRepository;
    @Autowired
    private ScaleRepository scaleRepository;
    @Autowired
    private KnowledgeMapper knowledgeMapper;

    private List<BaseNode> baseNodes;

    private Pageable pageable;
    /**
     * 处理症状节点相关的申请
     */
    @Override
    public Symptom addSymptom(Symptom symptom) {
        return symptomRepository.save(symptom);
    }

    @Override
    public Symptom findOneSymptom(long id) {
        return symptomRepository.findById(id).get();
    }

    @Override
    public List<Symptom> findSymptoms(String name) { return symptomRepository.findByNameContaining(name); }

    @Override
    public void deleteOneSymptom(long id) {
        symptomRepository.deleteById(id);
    }

    @Override
    public List<Symptom> selectAllSymptom(PageVo pageVo) {
        pageable = getPageable(pageVo);
        return IteratorUtils.toList(symptomRepository.findAll().iterator());
    }


    /**
     * 处理疾病节点相关的申请
     */
    @Override
    public Disease addDisease(Disease disease) {
        return diseaseRepository.save(disease);
    }

    @Override
    public Disease findOneDisease(long id) {
        return diseaseRepository.findById(id).get();
    }

    @Override
    public List<Disease> findDisease(String name) {
        return diseaseRepository.findByNameContaining(name);
    }

    @Override
    public void deleteOneDisease(long id) {diseaseRepository.deleteById(id); }

    @Override
    public List<Disease> selectAllDisease(PageVo pageVo) {
        pageable = getPageable(pageVo);
        return IteratorUtils.toList(diseaseRepository.findAll(pageable).iterator());
    }


    /**
     * 处理化验节点相关的申请
     */
    @Override
    public LIS addLIS(LIS lis) {
        return lisRepository.save(lis);
    }

    @Override
    public LIS findOneLIS(long id) {
        return lisRepository.findById(id).get();
    }

    @Override
    public List<LIS> findLIS(String name) { return lisRepository.findByNameContaining(name); }

    @Override
    public void deleteOneLIS(long id) {lisRepository.deleteById(id); }

    @Override
    public List<LIS> selectAllLIS(PageVo pageVo) {
        pageable = getPageable(pageVo);
        return IteratorUtils.toList(lisRepository.findAll().iterator());
    }


    @Override
    public List<Map<String, Object>> getRelationByNodes(RelationVo relationVo) {
        String s_name = relationVo.getStart_name();
        String s_type = relationVo.getStart_type();
        String e_name = relationVo.getEnd_name();
        String e_type = relationVo.getEnd_type();

        List<Map<String, Object>> list = extractPath(relationRepository.getRelationByNodes(s_name, s_type, e_name, e_type));
        return list;
    }

    @Override
    public List<Map<String, Object>> getOneLevelRelationByEndNode(RelationVo relationVo) {
        String e_name = relationVo.getEnd_name();
        String e_type = relationVo.getEnd_type();

        List<Map<String, Object>> list = extractPath(relationRepository.getOneLevelRelationByEndNode(e_name, e_type));
        return list;
    }

    @Override
    public Iterable<Map<String, Object>> getNodeByRelation(RelationVo relationVo) {
        String s_name = relationVo.getStart_name();
        String s_type = relationVo.getStart_type();
        String rel = relationVo.getRel_name();

        return relationRepository.getNodeByRelation(s_name, s_type, rel);
    }

    @Override
    public BaseNode updateNodeByName(AmendRelationVo amendRelationVo) {
        int id = amendRelationVo.getId();
        String old_name = amendRelationVo.getOld_start_name();
        String new_name = amendRelationVo.getNew_start_name();

        return baseNodeRepository.updateNodeByName(id, old_name, new_name);
    }

    @Override
    public List<Map<String, Object>> updateRelation(AmendRelationVo amendRelationVo) {
        List<Map<String, Object>> path = extractPath(knowledgeMapper.updateRelation(amendRelationVo));
        return path;
    }

    @Override
    public List<Map<String, Object>> getAllNode() {
        List<Map<String, Object>> ndlist = baseNodeRepository.getAllNode();
        return ndlist;
    }

    @Override
    public Map<String, Object> getHighRiskDisease(SearchData searchData) {
        List<FeatureRate> diags = searchData.getPushDiags();
        List<String> diseaseList = new ArrayList<>();
        if(diags != null && diags.size()>0){
            for (FeatureRate fe:diags) {
              String featureName = fe.getFeatureName();
              diseaseList.add(featureName);
            }
        }
        Map<String, Object> map = new HashMap<>();
        List<Map<String, Object>> list = baseNodeRepository.getHighRisk(diseaseList);
        for (Map hrmap:list) {
            String name = hrmap.get("name").toString();
            String risk = hrmap.get("risk").toString();
            map.put(name, risk);
        }
        return map;
    }

    @Override
    public ResponseData getDis(SearchData searchData) {
        ResponseData responseData = new ResponseData();
        List<String> newList = new ArrayList<>();
        List<String> inputList = new ArrayList<>();
        Map<String, Map<String, String>> inputs = searchData.getInputs();
        Set<String> ss = inputs.keySet();
        inputList.addAll(ss);
        //图谱计算是否有组合词
        List<Map<String, Object>> collectionWord = baseNodeRepository.getCollectionWord(inputList);
        if(collectionWord !=null && collectionWord.size()>0){
            for (Map<String, Object> word:collectionWord) {
                String jundgement = word.get("jundgement").toString();
                String standName = word.get("standName").toString();
                if("TRUE".equals(jundgement)){
                    inputList.add(standName);
                }
            }
        }
        List<Map<String, Object>> collectJYWord = baseNodeRepository.getCollectJYWord(inputList);
        if(collectJYWord !=null && collectJYWord.size()>0){
            for (Map<String, Object> word:collectJYWord) {
                String fild = word.get("fild").toString();
                String[] typeCollect = (String[])word.get("typeCollect");
                if(typeCollect.length == 1 && "近义词".equals(typeCollect[0].toString())){
                    List<Map<String, Object>> jyWord = baseNodeRepository.getJYWord(fild);
                    if(jyWord !=null && jyWord.size()>0){
                        for (Map<String, Object> ji:jyWord) {
                            String js = ji.get("js").toString();
                            inputList.remove(fild);
                            inputList.add(js);
                        }
                    }
                }
            }
        }
        newList.addAll(inputList);
        while (newList.size()>0){
            List<Map<String, Object>> condition = baseNodeRepository.getCondition(newList, inputList);
            newList.clear();
            if(condition !=null && condition.size()>0){
                for (Map<String, Object> cd:condition) {
                    String conditionName = cd.get("condition").toString();
                    String jundgement = cd.get("jundgement").toString();
                    if ("true".equals(jundgement)) {
                        newList.add(conditionName);
                        inputList.add(conditionName);
                    }
                }
            }
        }
        List<String> que = new ArrayList<>();
        List<Map<String, Object>> disease = baseNodeRepository.getDisease(inputList);
        if(disease !=null && disease.size()>0){
            for (Map<String, Object> d: disease) {
                String diseaseName = d.get("name").toString();
                String relationType = d.get("relationType").toString();
                if("确诊".equals(relationType) || "拟诊".equals(relationType)){
                    que.add(diseaseName);
                }
            }
        }
        List<FeatureRate> featureRates = new ArrayList<>();
        if(que !=null && que.size()>0){
            for (String quezhen:que) {
                FeatureRate featureRate = new FeatureRate();
                featureRate.setFeatureName(quezhen);
                featureRate.setExtraProperty("");
                if("低血糖反应".equals(quezhen) || "胃肠道不良反应".equals(quezhen)){
                    featureRate.setDesc("不良反应");
                }
                featureRate.setDesc("");
                featureRate.setRate("neo4j");
                featureRates.add(featureRate);
            }
        }
        String webDiag = searchData.getDiag();
        if(webDiag.trim()!=null){
            Map<String, Filnlly> mulDiseaseTreat = processTreat(webDiag, ss);
            responseData.setTreat(mulDiseaseTreat);
        }
        responseData.setDis(featureRates);
        return responseData;
    }

    @Override
    public Map<String, List<FeatureRate>> getLisPacs(SearchData searchData) {
        Map<String, List<FeatureRate>> lisPacsFeature = new HashMap<>();
        String webDiag = searchData.getDiag();
        List<String> webDiagList = Arrays.asList(webDiag.split(",|，|、"));
        List<FeatureRate> bigdataDiagFeature = searchData.getPushDiags();
        List<String> bigdataDiagList =new LinkedList<>();
        if(bigdataDiagFeature.size()>0){
            for (FeatureRate fe:bigdataDiagFeature) {
                if("neo4j".equals(fe.getRate())){
                    bigdataDiagList.add(fe.getFeatureName());
                    logger.info("图谱推出的诊断为: "+fe.getFeatureName());
                }else {
                    bigdataDiagList.add(fe.getFeatureName());
                    logger.info("大数据推出的诊断为: "+fe.getFeatureName());
                }
            }
        }
        for (String web: webDiagList ) {
            for (int i=0;i<bigdataDiagList.size();i++ ) {
                if(bigdataDiagList.get(i).equals(web)){
                    bigdataDiagList.remove(bigdataDiagList.get(i));
                }
            }
        }
        logger.info("界面诊断为: " + webDiagList);
        Map<String, Set<String>> weblisPacs =null;
        Map<String, Set<String>> biglisPacs =null;
        if(webDiagList !=null && webDiagList.size()>0){
            weblisPacs = processLisPacs(webDiagList);
        }
        if(bigdataDiagList !=null && bigdataDiagList.size()>0){
            biglisPacs = processLisPacs(bigdataDiagList);
        }
        Set<String> lis =null;
        Set<String> pacs =null;
        //如果界面有诊断
        if(weblisPacs !=null && weblisPacs.values().size()>0){
            lis = weblisPacs.get("LIS");
            pacs = weblisPacs.get("PACS");
            logger.info("界面有诊断的情况下,界面诊断推出的lis为: "+lis);
            logger.info("界面有诊断的情况下,界面诊断推出的pacs为: "+pacs);
            if(biglisPacs !=null && biglisPacs.values().size()>0){
                Set<String> bl = biglisPacs.get("LIS");
                Set<String>  bp= biglisPacs.get("PACS");
                logger.info("界面有诊断的情况下,推出诊断的lis为: "+bl);
                logger.info("界面有诊断的情况下,推出诊断的pacs为: "+bp);
                lis.addAll(bl);
                pacs.addAll(bp);
            }
        }else {
            lis = biglisPacs.get("LIS");
            pacs = biglisPacs.get("PACS");
            logger.info("界面无诊断的情况下,推出诊断的lis为: "+lis);
            logger.info("界面无诊断的情况下,推出诊断的lis为: "+pacs);
        }
        logger.info("推出的合并lis为: " + lis);
        logger.info("推出的合并pacs为: " + pacs);
        ArrayList<FeatureRate> lisFeature = new ArrayList<>();
        ArrayList<FeatureRate> pacsFeature = new ArrayList<>();
        for (String l:lis) {
            FeatureRate featureRate = new FeatureRate();
            featureRate.setFeatureName(l);
            lisFeature.add(featureRate);
        }
        for (String p:pacs) {
            FeatureRate featureRate = new FeatureRate();
            featureRate.setFeatureName(p);
            pacsFeature.add(featureRate);
        }
        lisPacsFeature.put("lisList",lisFeature);
        lisPacsFeature.put("pacsList",pacsFeature);
        return lisPacsFeature;
    }

    public Map<String, Set<String>> processLisPacs(List<String> webDiagList) {
        Map<String, Set<String>> lisPacsMap = new HashMap<>();
        List<String> lisArray = new ArrayList<>();
        List<String> pacsArray = new ArrayList<>();
        List<Map<String, Object>> lisPacsList = baseNodeRepository.getLisPacsList(webDiagList);
        if(lisPacsList !=null && lisPacsList.size()>0){
            for (Map<String, Object> f:lisPacsList) {
                String label = f.get("label").toString();
                String name = f.get("name").toString();
                if("LIS".equals(label)){
                    lisArray.add(name);
                }else if("PACS".equals(label)){
                    pacsArray.add(name);
                }
            }
        }
        lisPacsMap.put("LIS",processList(lisArray));
        lisPacsMap.put("PACS",processList(pacsArray));
        return lisPacsMap;
    }

    /**
     * 处理治疗
     * @param
     */
    public Map<String, Filnlly> processTreat(String webDisease,Set<String> inputFilds){
        Map<String, Filnlly> diagTreat = new HashMap<>();
        NumberFormat nf = NumberFormat.getPercentInstance();
        nf.setMinimumFractionDigits(0);//设置该百分比数字，保留2位小数;
        nf.setRoundingMode(RoundingMode.HALF_UP); //设置满5向上进位，即四舍五入;
        String[] diseaseArray = webDisease.split(",|，|、|;|:|；");
        List<String> diseaseList = Arrays.asList(diseaseArray);
        Map<String, List> diseFilds = new HashMap<>();
        for(int i=0;i<diseaseList.size();i++){
            List<String> fildsList = new ArrayList<>();//其他条件数组
            for (String fild:inputFilds) {
                fildsList.add(fild);
            }
            for (int j = 0; j < diseaseList.size(); j++) {
                if (i != j) {
                    fildsList.add(diseaseList.get(j));
                }
            }
            diseFilds.put(diseaseList.get(i), fildsList);
        }
        for (Map.Entry<String, List> disFild:diseFilds.entrySet()) {
            //获取诊断的药类和用药(排序好的)
            List<Map<String, Object>> treatMedicaton = baseNodeRepository.getTreatMedicaton(disFild.getKey());
            Map<String, LinkedHashMap<String, String>> contentMap = new LinkedHashMap<>();
            if(treatMedicaton != null && treatMedicaton.size()>0){
                for (Map<String, Object> tm:treatMedicaton) {
                    String drugs = tm.get("drugs").toString();
                    String medication = tm.get("medication").toString();
                    String rate = tm.get("rate").toString();
                    if (contentMap.containsKey(drugs)) {
                        LinkedHashMap<String, String> drugsMedicationLinkedHashMap = contentMap.get(drugs);
                        drugsMedicationLinkedHashMap.put(medication,rate);
                        contentMap.put(drugs,drugsMedicationLinkedHashMap);
                    }else {
                        LinkedHashMap<String, String> sa = new LinkedHashMap<>();
                        sa.put(medication,rate);
                        contentMap.put(drugs,sa);
                    }
                }
            }
            //药所属类型的大小类合并
            Map<String, String> shortLargeMap = new HashMap<>();
            List<Map<String, Object>> shortLarge = baseNodeRepository.getShortLarge(disFild.getKey());
            if(shortLarge != null && shortLarge.size()>0){
                for (Map<String, Object> sl:shortLarge) {
                    String aShort = sl.get("short").toString();
                    String large = sl.get("large").toString();
                    shortLargeMap.put(aShort,large+"("+aShort+")");
                }
            }
            //小类替换成大类
            Map<String, LinkedHashMap<String, String>> shortLargeMapConcat = new LinkedHashMap<>();
            for (Map.Entry<String, LinkedHashMap<String, String>> f : contentMap.entrySet()) {
                String drugs = f.getKey();
                if (shortLargeMap.get(drugs) != null) {
                    shortLargeMapConcat.put(shortLargeMap.get(drugs), contentMap.get(drugs));
                } else {
                    shortLargeMapConcat.put(drugs, contentMap.get(drugs));
                }
            }
            //处理慎用,忌用
            Map<String, String> medicationUseMap = new HashMap<>();//key:药名 value:禁忌(慎用)
            List<Map<String, Object>> shenJi = baseNodeRepository.getShenJiDrugsMedication(disFild.getKey(), disFild.getValue());
            if(shenJi != null  && shenJi.size()>0){
                for (Map<String, Object> sj:shenJi) {
                    String name = sj.get("name").toString();
                    String type = sj.get("type").toString();
                    medicationUseMap.put(name,type);//(名字,禁用/慎用)
                }
            }
            //大对象集合
            ArrayList<Drugs> drugsList = new ArrayList<Drugs>();
            DrugsPattern drugsPattern = new DrugsPattern();
            for (Map.Entry<String, LinkedHashMap<String, String>> w : shortLargeMapConcat.entrySet()) {
                int i = 0;
                int forbidden = 0;
                Drugs drugs = new Drugs();
                String drugsName = w.getKey();
                Map<String, String> bigSubDrugs = drugsPattern.bigSubDrugs(drugsName);
                drugs.setBigdrugsName(bigSubDrugs.get("big"));
                drugs.setSubdrugsName(bigSubDrugs.get("sub"));
                LinkedList<Medicition> medicitionsList = new LinkedList<>();
                LinkedHashMap<String, String> meditionRate = w.getValue();
                if ("忌用".equals(medicationUseMap.get(bigSubDrugs.get("big"))) || "忌用".equals(medicationUseMap.get(bigSubDrugs.get("sub")))) {
                    drugs.setDrugsForbidden(2);
                    for (Map.Entry<String, String> g : meditionRate.entrySet()) {
                        Medicition medicition = new Medicition();
                        String meditionName = g.getKey().trim();//药名
                        String rate = nf.format(Double.parseDouble(g.getValue().trim()));//百分比
                        medicition.setMedicitionName(meditionName);
                        medicition.setRate(rate);
                        if (i < 3) {
                            medicition.setIsShow(1);
                            i++;
                        } else {
                            medicition.setIsShow(0);
                        }
                        medicition.setForbidden(2);
                        medicitionsList.add(medicition);
                    }
                }else if("慎用".equals(medicationUseMap.get(bigSubDrugs.get("big"))) || "慎用".equals(medicationUseMap.get(bigSubDrugs.get("sub")))){
                    drugs.setDrugsForbidden(1);
                    for (Map.Entry<String, String> g : meditionRate.entrySet()) {
                        Medicition medicition = new Medicition();
                        String meditionName = g.getKey().trim();//药名
                        String rate = nf.format(Double.parseDouble(g.getValue().trim()));//百分比
                        medicition.setMedicitionName(meditionName);
                        medicition.setRate(rate);
                        if (i < 3) {
                            medicition.setIsShow(1);
                            i++;
                        } else {
                            medicition.setIsShow(0);
                        }
                        medicition.setForbidden(1);
                        medicitionsList.add(medicition);
                    }
                }else {
                    drugs.setDrugsForbidden(0);
                    for (Map.Entry<String, String> g : meditionRate.entrySet()) {
                        Medicition medicition = new Medicition();
                        String meditionName = g.getKey().trim();//药名
                        String rate = nf.format(Double.parseDouble(g.getValue().trim()));//百分比
                        medicition.setMedicitionName(meditionName);
                        medicition.setRate(rate);
                        if (i < 3) {
                            medicition.setIsShow(1);
                            i++;
                        } else {
                            medicition.setIsShow(0);
                        }
                        if ("忌用".equals(medicationUseMap.get(meditionName))) {
                            medicition.setForbidden(2);
                        } else if ("慎用".equals(medicationUseMap.get(meditionName))) {
                            medicition.setForbidden(1);
                        } else {
                            medicition.setForbidden(0);
                        }
                        medicitionsList.add(medicition);
                    }
                }
                drugs.setMedicitionsList(medicitionsList);
                drugsList.add(drugs);
            }
            Filnlly filnlly = new Filnlly();
            List<Indicators> indicatorsList1 = new ArrayList<>();
            List<Detail> detailList1 = new ArrayList<>();
            Detail detail1 = new Detail();
            detail1.setName("无");
            detail1.setValue(0);
            Detail detail2 = new Detail();
            detail2.setName("偶尔");
            detail2.setValue(0);
            Detail detail3 = new Detail();
            detail3.setName("频");
            detail3.setValue(0);
            detailList1.add(detail1);
            detailList1.add(detail2);
            detailList1.add(detail3);
            Indicators indicators1 = new Indicators();
            indicators1.setName("低血糖反应");
            indicators1.setControlType(2);
            indicators1.setDetails(detailList1);
            indicatorsList1.add(indicators1);

            List<Detail> detailList2 = new ArrayList<>();
            Detail detail11 = new Detail();
            detail11.setName("腹胀");
            detail11.setValue(0);
            Detail detail12 = new Detail();
            detail12.setName("厌食");
            detail12.setValue(0);
            Detail detail13 = new Detail();
            detail13.setName("嗳气");
            detail13.setValue(0);
            Detail detail14 = new Detail();
            detail14.setName("恶心");
            detail14.setValue(0);
            Detail detail15 = new Detail();
            detail15.setName("呕吐");
            detail15.setValue(0);
            detailList2.add(detail11);
            detailList2.add(detail12);
            detailList2.add(detail13);
            detailList2.add(detail14);
            detailList2.add(detail15);
            Indicators indicators11 = new Indicators();
            indicators11.setName("胃肠道不良反应");
            indicators11.setControlType(2);
            indicators11.setDetails(detailList2);
            indicatorsList1.add(indicators11);
            filnlly.setAdverseEvent(indicatorsList1);
            filnlly.setTreatment(drugsList);
            diagTreat.put(disFild.getKey(),filnlly);
        }
        return diagTreat;
    }

    @Override
    public List<Map<String, Object>> getRecommendItem(QueryVo queryVo) {
        List<Map<String, Object>> list = relationRepository.getRecommendItem(queryVo.getDiseaselist());
        return list;
    }

    @Override
    public Map<String, List<Map<String, Object>>> getScale(QueryVo queryVo) {
        Map<String, List<Map<String, Object>>> map = new HashMap<>();

        for (String name:queryVo.getScalelist()) {
            List<Map<String, Object>> list = scaleRepository.getScale(name);
            map.put(name, list);
        }

        return map;
    }


    public Pageable getPageable(PageVo pageVo) {
        if (pageVo.getSize() > 0) {
            if (pageVo.getSort_key() != null && pageVo.getSort_direct() != null) {
                if (pageVo.getSort_direct().toLowerCase().equals("asc")) {
                    pageable = PageRequest.of(pageVo.getCurrent(), pageVo.getSize(),
                            Sort.Direction.ASC, pageVo.getSort_key().trim());
                }
                else if (pageVo.getSort_direct().toLowerCase().equals("desc")) {
                    pageable = PageRequest.of(pageVo.getCurrent(), pageVo.getSize(),
                            Sort.Direction.DESC, pageVo.getSort_key().trim());
                }
                else {
                    pageable = PageRequest.of(pageVo.getCurrent(), pageVo.getSize());
                }
            }
            else {
                pageable = PageRequest.of(pageVo.getCurrent(), pageVo.getSize());
            }
        }
        else {
            pageable = PageRequest.of(0, 100);
        }
        return pageable;
    }

    public List<Map<String, Object>> extractPath(Iterable<Map<String, Object>> path) {
        Map<String, Object> item = new HashMap<>();
        List<Map<String, Object>> pathlist = new ArrayList<>();
        Map<String, Object> pathmap;

        Iterator<Map<String, Object>> iterator = path.iterator();
        while (iterator.hasNext()) {
            item = iterator.next();
            pathmap = new HashMap<>();
            if (item.get("P") != null) {
                Path p = (Path)item.get("P");
                pathmap = parsePath(p);
            }
            pathlist.add(pathmap);
        }

        return pathlist;
    }

    public Map<String, Object> parsePath(Path p) {
        Map<String, Object> path = new HashMap<>();
        path.put("start", new HashMap<>());
        path.put("end", new HashMap<>());
        Map<String, String> s = (Map)path.get("start");
        Map<String, String> e = (Map)path.get("end");
        Node start = p.start();
        Node end = p.end();
        s.put("label", start.labels().iterator().next());
        e.put("label", end.labels().iterator().next());
        String relation = p.relationships().iterator().next().type();
        path.put("relation", new HashMap<>());
        ((Map)path.get("relation")).put("type", relation);

        for (String prop:start.keys()) {
            s.put(prop, start.get(prop).toString().replaceAll("\"",""));
        }

        for (String prop:end.keys()) {
            e.put(prop, end.get(prop).toString().replaceAll("\"",""));
        }

        return path;
    }

    /**
     * 根据出现次数排序,去重
     * @param list
     * @return
     */
    public Set<String> processList(List<String> list) {
        Set<String> resultSet = new LinkedHashSet<>();
        Map<String, Integer> featuerInt = new HashMap<String, Integer>();
        for (String d : list) {
            if (featuerInt.containsKey(d)) {
                featuerInt.put(d, featuerInt.get(d) + 1);
            } else {
                featuerInt.put(d, 1);
            }
        }
        List<Map.Entry<String, Integer>> reslist = new ArrayList<Map.Entry<String, Integer>>(featuerInt.entrySet());
        Collections.sort(reslist, new Comparator<Map.Entry<String, Integer>>() {
            @Override
            public int compare(Map.Entry<String, Integer> o1, Map.Entry<String, Integer> o2) {
                return o2.getValue().compareTo(o1.getValue());
            }
        });
        for (Map.Entry<String, Integer> f : reslist
                ) {
            //排好序的set
            resultSet.add(f.getKey());
        }
        return resultSet;
    }

}
